// --- [ BMP085_main.c ] -------------------------------------------------------
//
//       tab = 3, 15.05.11

#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define EXTERN

#include "rs232.h"
#include "TWI_Master_c.h"
#include "global.h"
#include "bmp085_a.h"

// --- Prototypen --------------------------------------------------------------

//uint16_t BMP085_Read_Data(uint8_t adr);
uint16_t BMP085_Read_Temp(void);
uint16_t BMP085_Read_Pressure_16(void);
uint32_t BMP085_Read_Pressure_19(void);

// --- Globale Variablen -------------------------------------------------------

int16_t  ac1, ac2, ac3;
uint16_t ac4, ac5, ac6;
int16_t  b1, b2; 
int16_t  mb, mc, md;

// -----------------------------------------------------------------------------
void BMP085_Read_Calibration(void)
{
	ac1 = BMP085_Read_Data(0xAA);
	ac2 = BMP085_Read_Data(0xAC);
	ac3 = BMP085_Read_Data(0xAE);
	ac4 = BMP085_Read_Data(0xB0);
	ac5 = BMP085_Read_Data(0xB2);
	ac6 = BMP085_Read_Data(0xB4);

	b1  = BMP085_Read_Data(0xB6);	
	b2  = BMP085_Read_Data(0xB8);

	mb  = BMP085_Read_Data(0xBA);
	mc  = BMP085_Read_Data(0xBC);
	md  = BMP085_Read_Data(0xBE);
}

// -----------------------------------------------------------------------------
// liest einen 16-Bit Wert ab derjenigen Registeradresse, 
// die als Parameter bergeben wird

uint16_t BMP085_Read_Data(uint8_t adr)
{

twi_buffer[0] = BMP085_TWI_ADR;
twi_buffer[1] = adr;
TWI_MA_Start_Transceiver_With_Data(twi_buffer, 2);

twi_buffer[0]++;
TWI_MA_Start_Transceiver_With_Data(twi_buffer, 3);
TWI_MA_Get_Data_From_Transceiver(twi_buffer, 3);

return((twi_buffer[1] << 8) + twi_buffer[2]);
}

// -----------------------------------------------------------------------------
// liest den Temperaturwert uT ein
uint16_t BMP085_Read_Temp(void)
{

twi_buffer[0] = BMP085_TWI_ADR;
twi_buffer[1] = BMP085_CONTROL_REG;
twi_buffer[2] = BMP085_READ_TEMPERATURE;

TWI_MA_Start_Transceiver_With_Data(twi_buffer, 3);

#if EOC_ENABLE == TRUE
	while(BMP085_busy);
#else
	delay_10ms(1);						// Erforderliche Wartezeit 4.5ms
#endif

return( BMP085_Read_Data(BMP085_DATA_REG) );
}

#if READ_BIT == 19
// -----------------------------------------------------------------------------
// liest genau einen Luftdruckwert uP ein, jedoch mit mehreren internen Samples
// bei OSS > 0 muss ein 32-Bit-Wert zurckgegeben werden, da 19-Bit eingelesen werden
uint32_t BMP085_Read_Pressure_19(void)
{
uint32_t s = 0;

twi_buffer[0] = BMP085_TWI_ADR;
twi_buffer[1] = BMP085_CONTROL_REG;
twi_buffer[2] = BMP085_READ_PRESSURE + (BMP085_OSS << 6);
	
TWI_MA_Start_Transceiver_With_Data(twi_buffer, 3);

#if EOC_ENABLE == TRUE
	while(BMP085_busy);
#elif BMP085_OSS == 3
delay_10ms(3);						// Erforderliche Wartezeit 25.5ms
#else
delay_10ms(2);
#endif

twi_buffer[1] = BMP085_DATA_REG;
TWI_MA_Start_Transceiver_With_Data(twi_buffer, 2);
	
twi_buffer[0]++;
TWI_MA_Start_Transceiver_With_Data(twi_buffer, 4);
TWI_MA_Get_Data_From_Transceiver(twi_buffer, 4);

s = ((uint32_t)twi_buffer[1] << 16) + ((uint16_t)twi_buffer[2] << 8) + twi_buffer[3];

return (s >> (8 - BMP085_OSS));
}

#elif READ_BIT == 16
// -----------------------------------------------------------------------------
// liest genau einen Luftdruckwert uP ein, jedoch mit mehreren internen Samples
// es wird aber nur ein 16-Bit-Wert gelesen, die Bits 17 .. 19 werden ignoriert
// Dadurch kann das zustzliche Shiften in der Konvertierung entfallen
uint16_t BMP085_Read_Pressure_16(void)
{

twi_buffer[0] = BMP085_TWI_ADR;
twi_buffer[1] = BMP085_CONTROL_REG;
twi_buffer[2] = BMP085_READ_PRESSURE + (3 << 6);
	
TWI_MA_Start_Transceiver_With_Data(twi_buffer, 3);

#if EOC_ENABLE == TRUE
	while(BMP085_busy);
#elif BMP085_OSS == 3
delay_10ms(3);						// Erforderliche Wartezeit 25.5ms
#else
delay_10ms(2);
#endif

return( BMP085_Read_Data(BMP085_DATA_REG));
}
#else
	#error READ_BIT in bmp085_a.h nicht gesetzt
#endif

// -----------------------------------------------------------------------------
// liest uT und uP ein und rechnet Temperatur und Luftdruckwert um

void BMP085_Convert(long* temperature, long* pressure)
{
	int32_t ut, up;
	int32_t x1, x2, b5, b6, x3, b3, p;
	uint32_t b4, b7;

	uint8_t i;
	static uint16_t mt[8];				// Array zur Mittelwertbildung der Temperatur
	static uint8_t mt_i = 0;			// Index fr mt[]
	static uint8_t start_t = TRUE;

#if FLOATING_AVERAGE == TRUE
	static uint8_t  mp_i = 0;
	static uint32_t mp[AVERAGE_SAMPLES]; // Array zur Mittelwertbildung des Druckes
	static uint8_t start_p = TRUE;
#endif

// zuerst die Temperatur ermitteln, dazu den gleitenden Mittelwert aus 8 aufeinander
// folgenden Messungen bilden

// Um mgliche Irritationen beim Programmstart zu umgehen (7x falsche Temperaturwerte), 
// wird das Array fr die Mittelwertbildung beim Start mit Werten aufgefllt.
	if (start_t)
		{
		start_t = FALSE;
		while(mt_i < 7) mt[mt_i++] = BMP085_Read_Temp();
		}

	mt_i &= 7;									// berlauf des Index maskieren
	mt[mt_i++] =  BMP085_Read_Temp();
	
	ut = 0;
	for (i = 0; i < 8; i++) ut += mt[i];
	ut >>= 3;

#ifdef DEBUG	
	put_s("uT: ");								// Messwert Temperatur ausgeben
	put_s(ltoa(ut, cv, 10));
#endif	
	
	x1 = (ut - ac6) * ac5 >> 15;
	x2 = ((int32_t)mc << 11) / (x1 + md);
	b5 = x1 + x2;
	*temperature = (b5 + 8) >> 4;

// nun den Luftdruck ermitteln
//	dazu wird aus x Einzelmessungen der Mittelwert gebildet
// Abhngig von der gewhlten OSS-Einstellung hat der Chip selbst bereits
// einen Mittelwert aus mehreren Einzelmessungen zurckgegeben

	up = 0;				// Mittelwert aus x Einzelmessungen bilden
	i = SOFTWARE_SAMPLES;
	while(i--)
		{
#if READ_BIT == 19
		up += BMP085_Read_Pressure_19();
#elif READ_BIT == 16
		up += BMP085_Read_Pressure_16();
#else
	#error READ_BIT in bmp085_a.h nicht gesetzt
#endif
		}
	up /= SOFTWARE_SAMPLES;

#if FLOATING_AVERAGE	== TRUE		// einen zustzlichen gleitenden Mittelwert bilden
	if (start_p) 
		{
		start_p = FALSE;
		while(mp_i < (AVERAGE_SAMPLES - 1)) mp[mp_i++] = up;
		}
			
	mp_i &= (AVERAGE_SAMPLES - 1);// Index maskieren
	mp[mp_i++] = up;					// den Messwert ablegen

	up = 0;								// und nun den Mittelwert bilden
	for (i = 0; i < AVERAGE_SAMPLES; i++) up += mp[i];
	up /= AVERAGE_SAMPLES;
#endif

#ifdef DEBUG
	put_s(" uP: ");							// Messwert Druck ausgeben
	put_s(ltoa(up, cv, 10));
#endif

	b6 = b5 - 4000;
	x1 = (b2 * ((b6 * b6) >> 12)) >> 11;
	x2 = (ac2 * b6) >> 11;
	x3 = x1 + x2;

#if READ_BIT == 19
	b3 = (((((int32_t) ac1) * 4 + x3) << BMP085_OSS) + 2)>> 2;
#elif READ_BIT == 16
	b3 = (((((int32_t) ac1) * 4 + x3)) + 2)>> 2;
#else
	#error READ_BIT in bmp085_a.h nicht gesetzt
#endif	
	
	x1 = (ac3 * b6) >> 13;
	x2 = (b1 * ((b6 * b6) >> 12)) >> 16;
	x3 = ((x1 + x2) + 2) >> 2;
	b4 = (ac4 * (uint32_t) (x3 + 32768)) >> 15;

#if READ_BIT == 19	
	b7 = (((uint32_t) up - b3) * (50000 >> BMP085_OSS));
#elif READ_BIT == 16
	b7 = (((uint32_t) up - b3) * 50000);
#else
	#error READ_BIT in bmp085_a.h nicht gesetzt
#endif

	if (b7 < 0x80000000)	p = (b7 << 1) / b4;
	else                 p = (b7 / b4) << 1;
	x1 = (p >> 8) * (p >> 8);
	x1 = (x1 * 3038) >> 16;
	x2 = (-7357 * p) >> 16;
	*pressure = p + ((x1 + x2 + 3791) >> 4);

}

// --- [ eof ] -----------------------------------------------------------------
